@@ -61,24 +61,31 @@ def get_keyword_graph(
         pos=('ns', 'n', 'vn', 'v'),
         new=1.0,
         default_keyword=False,
+        deduplication=False,
         deny_word=True,
         fix_deny_fragment=True,
         max_len=10,
+        new_char=1,
+        min_count=3,
         score_negative=1.0,
         score_positive=-0.1,
 ) -> Tuple[List, List, Any]:
     """
     输入一组文本。提取关键词和边。
     :param texts: 一组文本
+
     :param top: 保留原始keyword的个数
     :param pos: 保留原始keyword的词性
     :param new: 新颖性得分权重
-    :param default_keyword:
+    :param default_keyword: False时从输入的文本中计算关键词，True时用实现计算好的（此时deduplication无效）。
+    :param deduplication: 对输入的文本去重
     :param deny_word: 是否向词表中加入所有否定词
     :param fix_deny_fragment: 是否补齐短语前的否定词
     :param max_len: 短语的最大长度
+    :param new_char: 短语必须包含至少new_char个新的字
+    :param min_count: 短语至少重复出现min_count次
     :param score_negative: 负向情感加分
-    :param score_positive: 正    向情感加分
+    :param score_positive: 正向情感加分
     :return: 节点表，边表，附加信息
     """
     if default_keyword:
@@ -86,6 +93,8 @@ def get_keyword_graph(
         keyword2score = load_pickle(path.join(DATAPATH, 'keyword.pkl'))
     else:
         # 不使用预先计算好的keyword
+        if deduplication:
+            texts = list(set(texts))
         concat_text = '\n'.join(texts)
         print('sentence:', len(texts), 'char', len(concat_text))
         keyword2score = get_keyword(concat_text, top=top, pos=pos)
@@ -150,6 +159,7 @@ def get_keyword_graph(
             if len(keypair2sentence[kp]) < 10:
                 keypair2sentence[kp].append(sentence)
             keypair_score_sum[kp] = keyword2score[sp_pos_select[i][0]] + keyword2score[sp_pos_select[i + 1][0]]
+    print('find keypair', len(keypair_score_sum))
     # keypair算特征
     keypair2times = {}
     keypair2distance_average = {}
@@ -187,18 +197,24 @@ def get_keyword_graph(
         all_feature.append([kp, keypair2sentence[kp], keypair_score_sum[kp], keypair2times[kp],
                             keypair2distance_average[kp], keypair2best_fragment[kp], keypair2sentiment[kp],
                             keypair2sentiment_negative[kp], keypair2rank_score[kp]])
-    # 筛选出没有显著重复的词
+    # 筛选短语
     all_feature.sort(key=lambda _: _[-1], reverse=True)
     keypair_selected = []
     keypair_selected2rank_score = {}
     char_selected = set()
     for kpl in all_feature:
         kp = kpl[0]
+        # 去除有显著重复的短语
         _char = set(keypair2best_fragment[kp])
-        if len(_char - char_selected) >= 2:
-            keypair_selected.append(kp)
-            keypair_selected2rank_score[kp] = keypair2rank_score[kp]
-            char_selected |= _char
+        if len(_char - char_selected) <= new_char:
+            continue
+        # 去除出现次数过少的短语
+        if keypair2times[kp] < min_count:
+            continue
+        keypair_selected.append(kp)
+        keypair_selected2rank_score[kp] = keypair2rank_score[kp]
+        char_selected |= _char
+    print('select keypair', len(keypair_selected))
     # 对入选的词做新颖性加分
     keypair_selected_new = []
     word_count = defaultdict(int)
@@ -218,7 +234,13 @@ def get_keyword_graph(
     # 整理出node表
     node = []
     for kp, score in keypair_selected_new:
-        node.append([keypair2best_fragment[kp], keypair2rank_score[kp], keypair2sentence[kp]])
+        node.append([
+            keypair2best_fragment[kp],
+            keypair2rank_score[kp],
+            keypair2sentence[kp],
+            keypair2times[kp],
+            keypair_score_sum[kp],
+        ])
     # 构建虚假的边集和
     relation = []
     len_node = len(node)
